require 'spec_helper'

describe Applitools::Eyes do
  init_eyes

  before do
    eyes.open(app_name: 'test_app', test_name: 'test_app_test')
    fake_session = Applitools::Session.new(1,"http://fake_url.com", false)
    eyes.session = fake_session
    eyes.agent_connector.stub(:stop_session).and_return(double("TestResults", mismatches: 0, missing: 0))
    eyes.agent_connector.stub(:match_window).and_return(true)
  end

  describe "#new" do
    context "doesn't have an apikey" do
      before do
        Applitools::Eyes.config[:apikey] = nil
      end
      it {expect {Applitools::Eyes.new }.to raise_error }
      after do
        Applitools::Eyes.config[:apikey] = 'YOUR_API_KEY_HERE'
      end
    end
    context "has an apikey url" do
      it {expect {Applitools::Eyes.new }.to_not raise_error }
    end
  end

  describe ".open" do
    context "already open" do
      it "aborts session" do
        eyes.should_receive(:abort_if_not_closed)
        expect {eyes.open}.to raise_error
      end
    end

  end

  describe ".abort_if_not_closed" do
    it "closes eyes" do
      expect(eyes).to be_open
      eyes.send(:abort_if_not_closed)
      expect(eyes).to_not be_open
    end

    it "clears session" do
      eyes.send(:abort_if_not_closed)
      expect(eyes.session).to be_nil
    end

    it "stops session" do
      expect(eyes.agent_connector).to receive(:stop_session)
      eyes.send(:abort_if_not_closed)
    end
  end

  describe ".close" do
    it "stops the session" do
      eyes.agent_connector.stub(:stop_session).and_return(double("TestResults", mismatches: 0, missing: 0))
      expect(eyes.agent_connector).to receive(:stop_session)
      expect(eyes.close)
    end

    context "new_session" do
      before { eyes.session.stub(:new_session?).and_return(true) }
      it "raises NewTestError" do
        eyes.session_start_info = double().as_null_object
        expect { eyes.close }.to raise_error(Applitools::NewTestError)
      end
    end

    context "not new_session" do
      before do
        eyes.stub(:session_start_info).and_return double().as_null_object 
        eyes.session.new_session = false
      end

      it "raises TestFailedError if results have mismatches" do
        eyes.agent_connector.stub(:stop_session).and_return(double("TestResults", mismatches: 2))
        expect { eyes.close }.to raise_error(Applitools::TestFailedError)
      end

      it "raises TestFailedError if results have missing" do
        eyes.agent_connector.stub(:stop_session).and_return(double("TestResults", mismatches: 0, missing: 2))
        expect { eyes.close }.to raise_error(Applitools::TestFailedError)
      end

      it "doesn't raise if old session without mismatches" do
        eyes.stub_chain(:agent_connector, :stop_session).and_return(double("TestResults", mismatches: 0, missing: 0))
        expect(eyes.close).to_not raise_error
      end
    end
  end

  describe ".check_window" do
    it "raises EyesError if eyes aren't open"
    context "no session" do 
      it "creates new session if no session exists"
      it "creates new MatchWindowTask"
    end

    context "results are as expected" do
      it "doesn't raise"
    end
    context "results are not as expected" do
      context "new session" do
        specify "next check_window will run once on time out"
      end
      context "not new session" do
        context "immediate failreports" do
          it "raises TestFailedError"
        end

        context "not immediate failreports" do
          it "doesn't raise"
        end
      end
    end
  end

  describe ".test" do
    it "opens and closes the eyes around the block execution" do
      eyes.should_receive(:open).ordered
      eyes.should_receive(:close).ordered
      eyes.test { "foo" }
    end

    it "always closes eyes even if there were exceptions" do
      eyes.stub(:open)
      eyes.should_receive(:abort_if_not_closed)
      eyes.test { raise Applitools::EyesError }
    end

    context "testing .test" do
      let!(:eyes) do
        Applitools::Eyes.new
      end
      it "catches triggers" do
        eyes.send(:abort_if_not_closed)
        eyes.test(app_name: 'my app1', test_name: 'my test') do |eyes, driver|
          driver.get "http://www.applitools.com"
          eyes.check_window("initial")
          driver.find_element(:css, "li.pricing a").click
          # return false unless user_inputs.map(&:to_hash) == [{"$type"=>"Applitools.Models.MouseTrigger, Core", :mouseAction=>1, :control=>{"$type"=>"Applitools.Utils.Geometry.MutableRegion, Core", :left=>555, :top=>0, :height=>69, :width=>50}, :location=>{:latitude=>25, :longitude=>34, :altitude=>nil}}]
          eyes.check_window("pricing page")
          el = driver.find_elements(:css, ".footer-row a").first
          driver.action.double_click(el).perform
          # return false unless user_inputs.map(&:to_hash) == [{"$type"=>"Applitools.Models.MouseTrigger, Core", :mouseAction=>3, :control=>{"$type"=>"Applitools.Utils.Geometry.MutableRegion, Core", :left=>0, :top=>0, :height=>115, :width=>376}, :location=>{:x=>115, :y=>376}}]
          eyes.check_window("in forms")
          other_el = driver.find_elements(:css, ".s2member-pro-paypal-registration-email").first
          driver.action.move_to(other_el).click(other_el).send_keys("text1",:space,"text2").perform
          # return false unless user_inputs.map(&:to_hash) == [{"$type"=>"Applitools.Models.MouseTrigger, Core", :mouseAction=>4, :control=>{"$type"=>"Applitools.Utils.Geometry.MutableRegion, Core", :left=>0, :top=>0, :height=>230, :width=>331}, :location=>{:x=>230, :y=>331}}, {"$type"=>"Applitools.Models.MouseTrigger, Core", :mouseAction=>1, :control=>{"$type"=>"Applitools.Utils.Geometry.MutableRegion, Core", :left=>0, :top=>0, :height=>230, :width=>331}, :location=>{:x=>230, :y=>331}}, {"$type"=>"Applitools.Models.TextTrigger, Core", :text=>"text1", :control=>{"$type"=>"Applitools.Utils.Geometry.MutableRegion, Core", :left=>230, :top=>331, :height=>367, :width=>35}}, {"$type"=>"Applitools.Models.TextTrigger, Core", :text=>"", :control=>{"$type"=>"Applitools.Utils.Geometry.MutableRegion, Core", :left=>230, :top=>331, :height=>367, :width=>35}}, {"$type"=>"Applitools.Models.TextTrigger, Core", :text=>"text2", :control=>{"$type"=>"Applitools.Utils.Geometry.MutableRegion, Core", :left=>230, :top=>331, :height=>367, :width=>35}}]
        end
      end
    end
  end
end
